"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.prefixTypeCast = exports.psqlTypeCast = void 0;
const typed_parser_1 = require("../typed-parser");
const ast_1 = require("../ast");
const token_1 = require("./token");
const utils_1 = require("./utils");
const optionalPrecision = (0, typed_parser_1.seq)((0, utils_1.optional)((0, utils_1.parenthesized)((0, typed_parser_1.match)('[0-9]+'))))((arg) => (arg === null ? '' : `(${arg})`));
const optionalDecimalPrecision = (0, typed_parser_1.seq)((0, utils_1.optional)((0, utils_1.parenthesized)((0, typed_parser_1.seq)((0, typed_parser_1.match)('[0-9]+'), (0, token_1.symbol)(','), (0, typed_parser_1.match)('[0-9]+'))((p, _, s) => `${p}, ${s}`))))((arg) => (arg === null ? '' : `(${arg})`));
const join = (...args) => args.join(' ').trim();
const intervalFieldNames = [
    'YEAR TO MONTH',
    'DAY TO HOUR',
    'DAY TO MINUTE',
    'DAY TO SECOND',
    'HOUR TO MINUTE',
    'HOUR TO SECOND',
    'MINUTE TO SECOND',
    'YEAR',
    'MONTH',
    'DAY',
    'HOUR',
    'MINUTE',
    'SECOND',
];
const optionalIntervalFields = (0, typed_parser_1.seq)((0, utils_1.optional)((0, typed_parser_1.oneOf)(...intervalFieldNames.map((field) => (0, token_1.sepExpectIdentifiers)(field)))))((arg) => (arg === null ? '' : arg));
const optionalTimeZoneModifier = (0, typed_parser_1.oneOf)((0, typed_parser_1.seq)((0, token_1.sepExpectIdentifiers)('with time zone'))(join), (0, typed_parser_1.seq)((0, utils_1.optional)((0, typed_parser_1.seq)((0, token_1.sepExpectIdentifiers)('without time zone'))(join)))((arg) => arg || ''));
const specialTypeCastTargetType = (syntax) => (0, typed_parser_1.oneOf)((0, typed_parser_1.seq)((0, token_1.sepExpectIdentifiers)('bit varying'), optionalPrecision)(join), (0, typed_parser_1.seq)((0, token_1.sepExpectIdentifiers)('bit'), optionalPrecision)(join), (0, typed_parser_1.seq)((0, token_1.sepExpectIdentifiers)('character varying'), optionalPrecision)(join), (0, token_1.sepExpectIdentifiers)('double precision'), (0, typed_parser_1.seq)((0, typed_parser_1.oneOf)((0, token_1.expectIdentifier)('numeric'), (0, token_1.expectIdentifier)('decimal')), optionalDecimalPrecision)(join), (0, typed_parser_1.seq)((0, typed_parser_1.oneOf)((0, token_1.expectIdentifier)('time'), (0, token_1.expectIdentifier)('timestamp')), optionalPrecision, optionalTimeZoneModifier)(join), (0, typed_parser_1.seq)((0, token_1.expectIdentifier)('interval'), 
// prefix typecast syntax doesn't support interval field names
syntax === 'prefix' ? (0, typed_parser_1.seqConst)('', typed_parser_1.noop) : optionalIntervalFields, optionalPrecision)(join));
exports.psqlTypeCast = (0, typed_parser_1.seq)((0, token_1.symbol)('::'), (0, typed_parser_1.oneOf)(specialTypeCastTargetType('psql'), token_1.identifier), (0, typed_parser_1.oneOf)((0, typed_parser_1.seqConst)('[]', (0, token_1.symbol)('['), (0, token_1.symbol)(']')), (0, typed_parser_1.seqConst)('', typed_parser_1._)))((_, id, arraySuffix) => id + arraySuffix);
/**
 * Typecasts of the form `type 'string'`
 *
 * Example: TIMEZONE (4) WITH TIME ZONE '2020-02-02T12:34:56.789123'
 *
 */
exports.prefixTypeCast = (0, typed_parser_1.seq)((0, typed_parser_1.oneOf)(specialTypeCastTargetType('prefix'), token_1.identifier), token_1.stringConstant)((targetType, value) => ast_1.Expression.createTypeCast(ast_1.Expression.createConstant(value), targetType));
